@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use base qw( Exporter );
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 use AnyEvent::RipeRedis::Error;
 
@@ -120,8 +120,9 @@ sub new {
   $self->{password} = $params{password};
   $self->{database}
       = defined $params{database} ? $params{database} : D_DB_INDEX;
+  $self->{utf8}      = exists $params{utf8}      ? $params{utf8}      : 1;
   $self->{reconnect} = exists $params{reconnect} ? $params{reconnect} : 1;
-  $self->{utf8}      = exists $params{utf8} ? $params{utf8} : 1;
+  $self->{handle_params}    = $params{handle_params} || {};
   $self->{on_connect}       = $params{on_connect};
   $self->{on_disconnect}    = $params{on_disconnect};
   $self->{on_connect_error} = $params{on_connect_error};
@@ -131,19 +132,11 @@ sub new {
   $self->min_reconnect_interval( $params{min_reconnect_interval} );
   $self->on_error( $params{on_error} );
 
-  my $hdl_params = $params{handle_params} || {};
-  foreach my $name ( qw( linger autocork ) ) {
-    if ( !exists $hdl_params->{$name} && defined $params{$name} ) {
-      $hdl_params->{$name} = $params{$name};
-    }
-  }
-  $self->{handle_params} = $hdl_params;
-
   $self->{_handle}           = undef;
   $self->{_connected}        = 0;
   $self->{_lazy_conn_state}  = $params{lazy};
   $self->{_auth_state}       = S_NEED_PERFORM;
-  $self->{_select_state}     = S_NEED_PERFORM;
+  $self->{_select_db_state}  = S_NEED_PERFORM;
   $self->{_ready}            = 0;
   $self->{_input_queue}      = [];
   $self->{_temp_queue}       = [];
@@ -166,7 +159,6 @@ sub info {
   my $cmd  = $self->_prepare_command( 'info', [@_] );
 
   weaken($self);
-
   my $on_reply = $cmd->{on_reply};
 
   $cmd->{on_reply} = sub {
@@ -241,11 +233,40 @@ sub eval_cached {
   my $self = shift;
   my $cmd  = $self->_prepare_command( 'evalsha', [@_] );
 
-  $cmd->{script} = $cmd->{args}[0];
-  unless ( exists $EVAL_CACHE{ $cmd->{script} } ) {
-    $EVAL_CACHE{ $cmd->{script} } = sha1_hex( $cmd->{script} );
+  my $script = $cmd->{args}[0];
+  unless ( exists $EVAL_CACHE{$script} ) {
+    $EVAL_CACHE{$script} = sha1_hex($script);
+  }
+  $cmd->{args}[0] = $EVAL_CACHE{$script};
+
+  {
+    weaken($self);
+    weaken( my $cmd = $cmd );
+
+    my $on_reply = $cmd->{on_reply};
+
+    $cmd->{on_reply} = sub {
+      my $reply = shift;
+      my $err   = shift;
+
+      if ( defined $err ) {
+        if ( $err->code == E_NO_SCRIPT ) {
+          $cmd->{kwd}     = 'eval';
+          $cmd->{args}[0] = $script;
+
+          $self->_push_write($cmd);
+
+          return;
+        }
+
+        $on_reply->( $reply, $err );
+
+        return;
+      }
+
+      $on_reply->($reply);
+    };
   }
-  $cmd->{args}[0] = $EVAL_CACHE{ $cmd->{script} };
 
   $self->_execute_command($cmd);
 
@@ -257,7 +278,6 @@ sub quit {
   my $cmd  = $self->_prepare_command( 'quit', [@_] );
 
   weaken($self);
-
   my $on_reply = $cmd->{on_reply};
 
   $cmd->{on_reply} = sub {
@@ -305,10 +325,11 @@ sub disconnect {
       if ( $self->{_txn_lock} ) {
         AE::postpone(
           sub {
-            my $err = AnyEvent::RipeRedis::Error->new(
-                qq{Command "$cmd->{kwd}" not allowed after "multi" command.}
-                    . ' First, the transaction must be finalized.',
-                E_OPRN_ERROR );
+            my $err = _new_error(
+              qq{Command "$cmd->{kwd}" not allowed after "multi" command.}
+                  . ' First, the transaction must be finalized.',
+              E_OPRN_ERROR
+            );
 
             $cmd->{on_reply}->( undef, $err );
           }
@@ -346,7 +367,7 @@ sub selected_database {
 sub on_error {
   my $self = shift;
 
-  if ( @_ ) {
+  if (@_) {
     my $on_error = shift;
 
     if ( defined $on_error ) {
@@ -447,13 +468,13 @@ sub _get_on_connect {
       $self->{_auth_state} = S_DONE;
     }
     if ( $self->{database} == 0 ) {
-      $self->{_select_state} = S_DONE;
+      $self->{_select_db_state} = S_DONE;
     }
 
     if ( $self->{_auth_state} == S_NEED_PERFORM ) {
       $self->_auth;
     }
-    elsif ( $self->{_select_state} == S_NEED_PERFORM ) {
+    elsif ( $self->{_select_db_state} == S_NEED_PERFORM ) {
       $self->_select_database;
     }
     else {
@@ -475,9 +496,10 @@ sub _get_on_connect_error {
   return sub {
     my $err_msg = pop;
 
-    my $err = AnyEvent::RipeRedis::Error->new(
-        "Can't connect to $self->{host}:$self->{port}: $err_msg",
-        E_CANT_CONN );
+    my $err = _new_error(
+      "Can't connect to $self->{host}:$self->{port}: $err_msg",
+      E_CANT_CONN
+    );
 
     $self->_disconnect($err);
   };
@@ -490,8 +512,7 @@ sub _get_on_rtimeout {
 
   return sub {
     if ( @{ $self->{_processing_queue} } ) {
-      my $err = AnyEvent::RipeRedis::Error->new( 'Read timed out.',
-          E_READ_TIMEDOUT );
+      my $err = _new_error( 'Read timed out.', E_READ_TIMEDOUT );
 
       $self->_disconnect($err);
     }
@@ -507,8 +528,8 @@ sub _get_on_eof {
   weaken($self);
 
   return sub {
-    my $err = AnyEvent::RipeRedis::Error->new(
-        'Connection closed by remote host.', E_CONN_CLOSED_BY_REMOTE_HOST );
+    my $err = _new_error( 'Connection closed by remote host.',
+        E_CONN_CLOSED_BY_REMOTE_HOST );
 
     $self->_disconnect($err);
   };
@@ -522,7 +543,7 @@ sub _get_handle_on_error {
   return sub {
     my $err_msg = pop;
 
-    my $err = AnyEvent::RipeRedis::Error->new( $err_msg, E_IO );
+    my $err = _new_error( $err_msg, E_IO );
     $self->_disconnect($err);
   };
 }
@@ -608,8 +629,8 @@ sub _get_on_read {
             }
           }
           else {
-            my $err = AnyEvent::RipeRedis::Error->new(
-                'Unexpected reply received.', E_UNEXPECTED_DATA );
+            my $err = _new_error( 'Unexpected reply received.',
+                E_UNEXPECTED_DATA );
 
             $self->_disconnect($err);
 
@@ -622,7 +643,7 @@ sub _get_on_read {
         my $curr_buf = $bufs[-1];
         if ( defined $err_code ) {
           unless ( ref($reply) ) {
-            $reply = AnyEvent::RipeRedis::Error->new( $reply, $err_code );
+            $reply = _new_error( $reply, $err_code );
           }
           $curr_buf->{err_code} = E_OPRN_ERROR;
         }
@@ -649,8 +670,6 @@ sub _prepare_command {
   my $kwd  = shift;
   my $args = shift;
 
-  weaken($self);
-
   my $cmd;
   if ( ref( $args->[-1] ) eq 'HASH' ) {
     $cmd = pop @{$args};
@@ -670,6 +689,8 @@ sub _prepare_command {
   $cmd->{args} = $args;
 
   unless ( defined $cmd->{on_reply} ) {
+    weaken($self);
+
     $cmd->{on_reply} = sub {
       my $reply = shift;
       my $err   = shift;
@@ -692,7 +713,7 @@ sub _execute_command {
     if ( defined $self->{_handle} ) {
       if ( $self->{_connected} ) {
         if ( $self->{_auth_state} == S_DONE ) {
-          if ( $self->{_select_state} == S_NEED_PERFORM ) {
+          if ( $self->{_select_db_state} == S_NEED_PERFORM ) {
             $self->_select_database;
           }
         }
@@ -726,10 +747,10 @@ sub _execute_command {
     else {
       AE::postpone(
         sub {
-          my $err = AnyEvent::RipeRedis::Error->new(
-              qq{Operation "$cmd->{kwd}" aborted:}
-                  .' No connection to the server.',
-              E_NO_CONN );
+          my $err = _new_error(
+            qq{Operation "$cmd->{kwd}" aborted: No connection to the server.},
+            E_NO_CONN
+          );
 
           $cmd->{on_reply}->( undef, $err );
         }
@@ -781,7 +802,6 @@ sub _auth {
   my $self = shift;
 
   weaken($self);
-
   $self->{_auth_state} = S_IN_PROGRESS;
 
   $self->_push_write(
@@ -800,7 +820,7 @@ sub _auth {
 
         $self->{_auth_state} = S_DONE;
 
-        if ( $self->{_select_state} == S_NEED_PERFORM ) {
+        if ( $self->{_select_db_state} == S_NEED_PERFORM ) {
           $self->_select_database;
         }
         else {
@@ -818,8 +838,7 @@ sub _select_database {
   my $self = shift;
 
   weaken($self);
-
-  $self->{_select_state} = S_IN_PROGRESS;
+  $self->{_select_db_state} = S_IN_PROGRESS;
 
   $self->_push_write(
     { kwd  => 'select',
@@ -829,14 +848,15 @@ sub _select_database {
         my $err = $_[1];
 
         if ( defined $err ) {
-          $self->{_select_state} = S_NEED_PERFORM;
+          $self->{_select_db_state} = S_NEED_PERFORM;
           $self->_abort($err);
 
           return;
         }
 
-        $self->{_select_state} = S_DONE;
-        $self->{_ready}        = 1;
+        $self->{_select_db_state} = S_DONE;
+        $self->{_ready}           = 1;
+
         $self->_flush_input_queue;
       },
     }
@@ -886,32 +906,24 @@ sub _process_error {
   my $cmd = shift @{ $self->{_processing_queue} };
 
   unless ( defined $cmd ) {
-    my $err = AnyEvent::RipeRedis::Error->new(
-        "Don't know how process error message. Processing queue is empty.",
-        E_UNEXPECTED_DATA );
+    my $err = _new_error(
+      q{Don't know how process error message. Processing queue is empty.},
+      E_UNEXPECTED_DATA
+    );
 
     $self->_disconnect($err);
 
     return;
   }
 
-  if ( $err_code == E_NO_SCRIPT && defined $cmd->{script} ) {
-    $cmd->{kwd} = 'eval';
-    $cmd->{args}[0] = delete $cmd->{script};
-
-    $self->_push_write($cmd);
-
-    return;
-  }
-
   if ( ref($reply) ) {
-    my $err = AnyEvent::RipeRedis::Error->new(
+    my $err = _new_error(
         qq{Operation "$cmd->{kwd}" completed with errors.}, $err_code );
 
     $cmd->{on_reply}->( $reply, $err );
   }
   else {
-    my $err = AnyEvent::RipeRedis::Error->new( $reply, $err_code );
+    my $err = _new_error( $reply, $err_code );
     $cmd->{on_reply}->( undef, $err );
   }
 
@@ -925,10 +937,11 @@ sub _process_message {
   my $cmd = $self->{_channels}{ $msg->[1] };
 
   unless ( defined $cmd ) {
-    my $err = AnyEvent::RipeRedis::Error->new(
-        q{Don't know how process published message.}
-            . qq{ Unknown channel or pattern "$msg->[1]".},
-        E_UNEXPECTED_DATA );
+    my $err = _new_error(
+      q{Don't know how process published message.}
+          . qq{ Unknown channel or pattern "$msg->[1]".},
+      E_UNEXPECTED_DATA
+    );
 
     $self->_disconnect($err);
 
@@ -948,9 +961,10 @@ sub _process_success {
   my $cmd = $self->{_processing_queue}[0];
 
   unless ( defined $cmd ) {
-    my $err = AnyEvent::RipeRedis::Error->new(
-        "Don't know how process reply. Processing queue is empty.",
-        E_UNEXPECTED_DATA );
+    my $err = _new_error(
+      q{Don't know how process reply. Processing queue is empty.},
+      E_UNEXPECTED_DATA
+    );
 
     $self->_disconnect($err);
 
@@ -1004,11 +1018,11 @@ sub _disconnect {
     $self->{_handle}->destroy;
     undef $self->{_handle};
   }
-  $self->{_connected}    = 0;
-  $self->{_auth_state}   = S_NEED_PERFORM;
-  $self->{_select_state} = S_NEED_PERFORM;
-  $self->{_ready}        = 0;
-  $self->{_txn_lock}     = 0;
+  $self->{_connected}       = 0;
+  $self->{_auth_state}      = S_NEED_PERFORM;
+  $self->{_select_db_state} = S_NEED_PERFORM;
+  $self->{_ready}           = 0;
+  $self->{_txn_lock}        = 0;
 
   $self->_abort($err);
 
@@ -1038,8 +1052,8 @@ sub _abort {
   $self->{_pchannel_cnt}     = 0;
 
   if ( !defined $err && @unfin_cmds ) {
-    $err = AnyEvent::RipeRedis::Error->new(
-        'Connection closed by client prematurely.', E_CONN_CLOSED_BY_CLIENT );
+    $err = _new_error( 'Connection closed by client prematurely.',
+        E_CONN_CLOSED_BY_CLIENT );
   }
 
   if ( defined $err ) {
@@ -1055,8 +1069,10 @@ sub _abort {
 
     if ( %channels && $err_code != E_CONN_CLOSED_BY_CLIENT ) {
       foreach my $name ( keys %channels ) {
-        my $err = AnyEvent::RipeRedis::Error->new(
-            qq{Subscription to channel "$name" lost: $err_msg}, $err_code );
+        my $err = _new_error(
+          qq{Subscription to channel "$name" lost: $err_msg},
+          $err_code
+        );
 
         my $cmd = $channels{$name};
         $cmd->{on_reply}->( undef, $err );
@@ -1064,8 +1080,8 @@ sub _abort {
     }
 
     foreach my $cmd (@unfin_cmds) {
-      my $err = AnyEvent::RipeRedis::Error->new(
-          qq{Operation "$cmd->{kwd}" aborted: $err_msg}, $err_code );
+      my $err = _new_error( qq{Operation "$cmd->{kwd}" aborted: $err_msg},
+          $err_code );
 
       $cmd->{on_reply}->( undef, $err );
     }
@@ -1074,6 +1090,10 @@ sub _abort {
   return;
 }
 
+sub _new_error {
+  return AnyEvent::RipeRedis::Error->new(@_);
+}
+
 sub AUTOLOAD {
   our $AUTOLOAD;
   my $method = $AUTOLOAD;
@@ -1108,7 +1128,7 @@ sub DESTROY {
     );
 
     foreach my $cmd ( @unfin_cmds ) {
-      warn "Operation \"$cmd->{kwd}\" aborted:"
+      warn qq{Operation "$cmd->{kwd}" aborted:}
           . " Client object destroyed prematurely.\n";
     }
   }
@@ -1219,10 +1239,10 @@ after connection.
 
 =item database => $index
 
-Database index. If the index is specified, the client is switched to
-the specified database after connection. You can also switch to another database
-after connection by using C<SELECT> command. The client remembers last selected
-database after reconnection.
+Database index. If the index is specified, the client switches to the specified
+database after connection. You can also switch to another database after
+connection by using C<SELECT> command. The client remembers last selected
+database after reconnection and switches to it automaticaly.
 
 The default database index is C<0>.
 
@@ -1235,12 +1255,10 @@ Enabled by default.
 
 =item connection_timeout => $fractional_seconds
 
-Timeout, within which the client will be wait the connection establishment to
-the Redis server. If the client could not connect to the server after specified
-timeout, the C<on_error> or C<on_connect_error> callback is called. In case
-when C<on_error> callback is called, C<E_CANT_CONN> error code is passed to
-callback in the second argument. The timeout specifies in seconds and can
-contain a fractional part.
+Specifies connection timeout. If the client could not connect to the server
+after specified timeout, the C<on_connect_error> callback is called with the
+C<E_CANT_CONN> error, or if it not specified, the C<on_error> callback is
+called. The timeout specifies in seconds and can contain a fractional part.
 
   connection_timeout => 10.5,
 
@@ -1248,10 +1266,9 @@ By default the client use kernel's connection timeout.
 
 =item read_timeout => $fractional_seconds
 
-Timeout, within which the client will be wait a response on a command from the
-Redis server. If the client could not receive a response from the server after
-specified timeout, the client close connection and call C<on_error> callback
-with the C<E_READ_TIMEDOUT> error code. The timeout is specifies in seconds
+Specifies read timeout. If the client could not receive a reply from the server
+after specified timeout, the client close connection and call the C<on_error>
+callback with the C<E_READ_TIMEDOUT> error. The timeout is specifies in seconds
 and can contain a fractional part.
 
   read_timeout => 3.5,
@@ -1279,8 +1296,8 @@ Enabled by default.
 
 =item min_reconnect_interval => $fractional_seconds
 
-If the parameter is specified, the client will try to reconnect not often than
-through this interval.
+If the parameter is specified, the client will try to reconnect not often, than
+after this interval.
 
   min_reconnect_interval => 5,
 
@@ -1288,7 +1305,8 @@ Not set by default.
 
 =item handle_params => \%params
 
-Specifies L<AnyEvent::Handle> parameters.
+Specifies L<AnyEvent::Handle> parameters. Enabling of the C<autocork> parameter
+can improve perfomance.
 
   handle_params => {
     autocork => 1,
@@ -1320,10 +1338,10 @@ Not set by default.
 =item on_error => $cb->( $err )
 
 The C<on_error> callback is called when occurred an error, which was affected
-on entire client (e. g. connection error or authentication error). Also the
-C<on_error> callback is called on command errors if command callback is not
-specified. If C<on_error> callback is not specified, the client just print an
-error messages to C<STDERR>.
+on whole client (e. g. connection error or authentication error). Also the
+C<on_error> callback is called on command errors if the command callback is not
+specified. If the C<on_error> callback is not specified, the client just print
+an error messages to C<STDERR>.
 
 =back
 
@@ -1331,18 +1349,16 @@ error messages to C<STDERR>.
 
 =head2 <command>( [ @args ] [, $cb->( $reply, $err ) ] )
 
-The callback in command method is optional and can be passed as last argument.
-If the callback specified, the reply is passed to the callback as first
-argument. If any error occurred during command execution, the error object is
-passed to the callback as second argument. If the callback in command method is
-not specified and error occurred, the C<on_error> callback of the client is
-called.
-
-Error object is the instance of the class L<AnyEvent::RipeRedis::Error> and has
-two methods: C<message()> to get error message and C<code()> to get error code.
-
 The full list of the Redis commands can be found here: L<http://redis.io/commands>.
 
+The reply to the command is passed to the callback in first argument. If any
+error occurred during command execution, the error object is passed to the
+callback in second argument. Error object is the instance of the class
+L<AnyEvent::RipeRedis::Error>.
+
+The command callback is optional. If it is not specified and any error
+occurred, the C<on_error> callback of the client is called.
+
   $redis->set( 'foo', 'string' );
 
   $redis->get( 'foo',
@@ -1377,9 +1393,26 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
     }
   );
 
+You can execute multi-word commands like this:
+
+  $redis->client_getname(
+    sub {
+      my $reply = shift;
+      my $err   = shift;
+
+      if ( defined $err ) {
+        # error handling...
+
+        return;
+      }
+
+      print "$reply\n";
+    }
+  );
+
 =head1 TRANSACTIONS
 
-The detailed information abount the Redis transactions can be found here:
+The detailed information about the Redis transactions can be found here:
 L<http://redis.io/topics/transactions>.
 
 =head2 multi( [ $cb->( $reply, $err ) ] )
@@ -1393,9 +1426,9 @@ Executes all previously queued commands in a transaction and restores the
 connection state to normal. When using C<WATCH>, C<EXEC> will execute commands
 only if the watched keys were not modified.
 
-If during a transaction at least one command fails, to the callback of C<exec()>
-method will be passed error object and the reply will be contain nested error
-objects for every failed command.
+If during a transaction at least one command fails, to the callback will be
+passed error object, and the reply will be contain nested error objects for
+every failed command.
 
   $redis->multi();
   $redis->set( 'foo', 'string' );
@@ -1457,14 +1490,17 @@ L<http://redis.io/topics/pubsub>
 
 =head2 subscribe( @channels, ( $cb | \%cbs ) )
 
-Subscribes the client to the specified channels. Method can accept two
-callbacks: C<on_reply> and C<on_message>. The C<on_reply> callback is called
-once when subscription to all specified channels will be activated. In first
-argument to the callback is passed the number of channels we are currently
-subscribed. If subscription to specified channels was lost, the C<on_reply>
-callback is called with the error object in the second argument. The
-C<on_message> callback is called on every published message. If method is
-called with one callback, it will be act as C<on_message> callback.
+Subscribes the client to the specified channels.
+
+Method can accept two callbacks: C<on_reply> and C<on_message>. The C<on_reply>
+callback is called when subscription to all specified channels will be
+activated. In first argument to the callback is passed the number of channels
+we are currently subscribed. If subscription to specified channels was lost,
+the C<on_reply> callback is called with the error object in the second argument.
+
+The C<on_message> callback is called on every published message. If the
+C<subscribe> method is called with one callback, this callback will be act as
+C<on_message> callback.
 
   $redis->subscribe( qw( foo bar ),
     { on_reply => sub {
@@ -1545,7 +1581,7 @@ Posts a message to the given channel.
 
 Unsubscribes the client from the given channels, or from all of them if none
 is given. In first argument to the callback is passed the number of channels we
-are currently subscribed or zero if we are unsubscribed from all channels.
+are currently subscribed or zero if we were unsubscribed from all channels.
 
   $redis->unsubscribe( qw( foo bar ),
     sub {
@@ -1600,7 +1636,7 @@ Redis 2.6 and higher support execution of Lua scripts on the server side.
 To execute a Lua script you can send one of the commands C<EVAL> or C<EVALSHA>,
 or use the special method C<eval_cached()>.
 
-=head2 eval_cached( $script, $numkeys [, @keys ] [, @args ] [, $cb->( $reply, $err ) ] ] );
+=head2 eval_cached( $script, $keys_num [, @keys ] [, @args ] [, $cb->( $reply, $err ) ] ] );
 
 When you call the C<eval_cached()> method, the client first generate a SHA1
 hash for a Lua script and cache it in memory. Then the client optimistically
@@ -1633,8 +1669,8 @@ Be care, passing a different Lua scripts to C<eval_cached()> method every time
 cause memory leaks.
 
 If Lua script returns multi-bulk reply with at least one error reply, to the
-callback of C<eval_cached> method will be passed error object and the reply
-will be contain nested error objects.
+callback will be passed error object, and the reply will be contain nested
+error objects.
 
   $redis->eval_cached( "return { 'foo', redis.error_reply( 'Error.' ) }", 0,
     sub {
@@ -1667,9 +1703,9 @@ will be contain nested error objects.
 
 =head1 ERROR CODES
 
-Error codes can be used for programmatic handling of errors.
-AnyEvent::RipeRedis provides constants of error codes, which can be imported
-and used in expressions.
+Every error object, passed to callback, contain error code, which can be used
+for programmatic handling of errors. AnyEvent::RipeRedis provides constants for
+error codes. They can be imported and used in expressions.
 
   use AnyEvent::RipeRedis qw( :err_codes );
 
